#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile)
runiraf(){
  TMPPATH=/tmp/$USER
  if [ ! -d $TMPPATH ];then
   mkdir $TMPPATH
  fi

  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2. #" 
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep DATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter DATA not defined!!"
 exit 1
fi

if [ `env | grep LOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter LOG not defined!!"
 exit 1
fi

TMPPATH=/tmp/$USER
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

if [ $# != 5 ]; then
    echo "Usage : "$0" (YYMMDD) (Detector ID) (Start No.) (No. of Frames) (Header Name)"
    exit 1
fi

DATE=$1
DETECTOR=$2
IS=$3
N=$4
HEAD=$5

CLCOM=$TMPPATH/cl.com
CLLOG=log/mask.$HEAD.`date +%y%m%d%H%M%S`.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`

echo Log will be saved as $CLLOG
echo IRAF commands will be saved as $CLLOGCOM

# Check Architecture
ARCH=`uname`
#uname -a >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi


SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits
MSKY=mask${HEAD}_sky${DETECTOR}.fits
MSKYFLAT=mask${HEAD}_skyflat${DETECTOR}.fits

MASKBASE=maskbase.fits

DATAPATH=$DATA/wfcam${DETECTOR}/${DATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
esac

gawk -v path=$DATAPATH -v date=$DATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl

gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

echo "## Detect Objects in $FINAL"
echo "## Detect Objects in $FINAL" >> $CLLOG

cat <<EOF > $CLCOM
imdel tmp.imh,mtmp.imh
magnify $FINAL tmp.imh 0.5 0.5
median tmp.imh mtmp.imh 3 3
del tmp.dat
imstati tmp.imh[513:1536,513:1536] format- > tmp.dat
logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

IMSTATUPPER=`gawk '{print($3+$4*3)}' tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' tmp.dat`

cat << EOF > $CLCOM
del tmpstd.dat
imstati tmp.imh[513:1536,513:1536] format- fields=stddev upper=$IMSTATUPPER lower=$IMSTATLOWER > tmpstd.dat
del tmpsky.dat
imstati tmp.imh[513:1536,513:1536] format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > tmpsky.dat
logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

SIGMA=`cat tmpstd.dat`
SKYLV=`cat tmpsky.dat`
DITHX=`gawk 'BEGIN{xmax=-10000;ymax=-10000;xmin=10000;ymin=10000}{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}END{print(int((xmax-xmin)/2)+5)}' offset.tbl`
DITHY=`gawk 'BEGIN{xmax=-10000;ymax=-10000;xmin=10000;ymin=10000}{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}END{print(int((ymax-ymin)/2)+5)}' offset.tbl`
DITHX2=`expr $DITHX + $DITHX`
DITHY2=`expr $DITHY + $DITHY`

cat << EOF > $CLCOM
 del tmp.coo
 noao
 digiphot
 apphot
 imshift mtmp.imh mtmp.imh $DITHX $DITHY boundary=constant constan=$SKYLV
 imshift mtmp.imh mtmp.imh -$DITHX2 0 boundary=constant constan=$SKYLV
 imshift mtmp.imh mtmp.imh 0 -$DITHY2 boundary=constant constan=$SKYLV
 imshift mtmp.imh mtmp.imh $DITHX $DITHY boundary=constant constan=$SKYLV
 daofind mtmp.imh output=tmp.coo fwhmpsf=6 thresho=4 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
 disp tmp.imh 1 fi+
 tvmark 1 tmp.coo number+
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

# Need Photometry of each objects?

echo "## Create mask frames"
echo "## Create mask frames" >> $CLLOG

sed '/#/d' tmp.coo | gawk '{print($1,$2)}'> tmp2.coo
gawk 'BEGIN{xmin=10000;ymin=10000}{if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;x[NR]=$1;y[NR]=$2}END{for(i=1;i<=NR;i++)print((x[i]-xmin)/2,(y[i]-ymin)/2)}' offset.tbl > zoffset.tbl

cat << EOF > $CLCOM
 imdel $MASKBASE
 imarith $SKYFLAT * 0 $MASKBASE
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

\rm mask.tbl
for dnumber in `gawk '{print(NR)}' offset.tbl`;do
 dnumber2=`echo $dnumber | gawk '{printf("%03d",$1)}'`
# echo $dnumber2
 MASKFILE=$MASKPATH/$HEAD_${dnumber2}.pl
 echo $MASKFILE >> mask.tbl
 LFILE=`sed -n ${dnumber}p ldata.tbl`
 DX=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($1)}'`
 DY=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($2)}'`
# echo $dnumber2 $DX $DY
 gawk -v dx=$DX -v dy=$DY '{x=$1-dx;y=$2-dy;if(x>0 && y>0 && x<2049 && y<2049)print(x,y)}' tmp2.coo > tmp3.coo
 cat << EOF > $CLCOM
  imdel $MASKFILE
  imedit $MASKBASE $MASKFILE cursor=tmp3.coo display- radius=6 value=1 sigma=0 default="e "
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 
done


#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'


