#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile)
runiraf(){
  TMPPATH=/tmp/$USER
  if [ ! -d $TMPPATH ];then
   mkdir $TMPPATH
  fi

  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2. #" 
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep DATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter DATA not defined!!"
 exit 1
fi

if [ `env | grep LOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter LOG not defined!!"
 exit 1
fi

#Check command arguments
if [ $# != 5 ]; then
    echo "Usage : "$0" (YYMMDD) (Detector ID) (Start No.) (No. of Frames) (Header Name)"
    exit 1
fi

DATE=$1
DETECTOR=$2
IS=$3
N=$4
HEAD=$5


# setup filenames
TMPPATH=/tmp/$USER
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

CLCOM=$TMPPATH/cl.com
CLLOG=log/mask.$HEAD.`date +%y%m%d%H%M%S`.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`

echo Log will be saved as $CLLOG
echo IRAF commands will be saved as $CLLOGCOM

# Check Architecture
ARCH=`uname`
#uname -a >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits
MSKY=mask${HEAD}_sky${DETECTOR}.fits
MSKYFLAT=mask${HEAD}_skyflat${DETECTOR}.fits

MASKBASE=maskbase.fits

DATAPATH=$DATA/wfcam${DETECTOR}/${DATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
esac

gawk -v path=$DATAPATH -v date=$DATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

# Re-acquire raw frames
cat << EOF > $CLCOM
 imdel @ldata.tbl
 imdel @tdata.tbl >> $CLCOM
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s\n",$1,$2,dark)}' | csh 1>> $CLLOG 

gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM

cat << EOF >> $CLCOM
 del tmp.dat
 imstati @ldata-1.tbl upper=5500 lower=2500 format- > tmp.dat
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]/s)}' tmp.dat > tmp2.dat

cat << EOF > $CLCOM
 imarith @ldata.tbl / @tmp2.dat @ldata.tbl
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 


#register mask frame
\rm $CLCOM
ndith=`gawk 'END{print(NR)}' mask.tbl`
for dnumber in `gawk '{print(NR)}' ldata.tbl`;do
 dnumber2=`echo $dnumber | gawk -v ndith=$ndith '{print(($1-1)%ndith+1)}'`
 echo $dnumber $dnumber2  > tmpnumlist
 MASKFILE=`sed -n ${dnumber2}p mask.tbl`
 LFILE=`sed -n ${dnumber}p ldata.tbl`
 echo hedit $LFILE bpm $MASKFILE add+ ver- >> $CLCOM
done
echo logout >> $CLCOM

runiraf $CLCOM $CLLOG $CLLOGCOM 

#combine
cat << EOF > $CLCOM
 del $MSKY
 imcombine @ldata.tbl $MSKY combine=median reject=sigclip outtype=real masktype=goodvalue maskvalue=0
 disp $MSKY 2 fi+

 del tmp.dat
 imstati $MSKY format- > tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

IMSTATUPPER=`gawk '{print($3+$4*3)}' tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' tmp.dat`

echo $IMSTATUPPER

cat <<EOF > $CLCOM
del tmp.dat
imstati $MSKY format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > tmp.dat
imdel $MSKYFLAT
imarith $MSKY / @tmp.dat $MSKYFLAT

# Fix badpix

imdel tmp.imh
imcopy $MSKYFLAT tmp.imh
imreplace tmp.imh -1 upper=0.01 
imreplace tmp.imh 0 lower=0
imreplace tmp.imh 1 upper=-0.5
fixpix $MSKYFLAT tmp.imh 
logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

echo "Sky frame saved as $MSKY"
echo "Skyflat frame saved as $MSKYFLAT"


#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo Log saved as $CLLOG
echo IRAF commands saved as $CLLOGCOM

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'


