#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile) (output commandlist)
runiraf(){
  TMPPATH=/tmp/$USER
  if [ ! -d $TMPPATH ];then
   mkdir $TMPPATH
  fi

  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2. #" 
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep DATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter DATA not defined!!"
 exit 1
fi

if [ `env | grep LOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter LOG not defined!!"
 exit 1
fi

TMPPATH=/tmp/$USER
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

if [ $# != 5 ]; then
    echo "Usage : "$0" (YYMMDD) (Detector ID) (Start No.) (No. of Frames) (Header Name)"
    exit 1
fi

DATE=$1
DETECTOR=$2
IS=$3
N=$4
HEAD=$5

CLCOM=$TMPPATH/cl.com
CLLOG=log/mask.$HEAD.`date +%y%m%d%H%M%S`.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`

echo Log will be saved as $CLLOG
echo IRAF commands will be saved as $CLLOGCOM

# Check Architecture
ARCH=`uname`
#uname -a >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

SKYFLAT=flat.fits
#SKYFLAT=${HEAD}_flat${DETECTOR}.fits
#SKY=${HEAD}_sky${DETECTOR}.fits
DARK=dark${DETECTOR}.fits
BADPIX=badpix${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits

DATAPATH=$DATA/wfcam${DETECTOR}/${DATE}

TDATAPATH=./tdata
LDATAPATH=./ldata

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $LOG > offset.tbl ;;
esac

gawk -v path=$DATAPATH -v date=$DATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl

gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

cat << EOF > $CLCOM
 imdel @ldata.tbl
 imdel @tdata.tbl >> $CLCOM
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

paste data.tbl ldata.tbl | gawk -v flat=$SKYFLAT -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s %s\n",$1,$2,dark,flat)}' | csh 1>> $CLLOG

gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl > $CLCOM
cat << EOF >> $CLCOM
 del tmp.dat
 imstati @ldata-1.tbl upper=5500 lower=2500 format- > tmp.dat
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]-s)}' tmp.dat > tmp2.dat

echo imarith @ldata.tbl - @tmp2.dat @ldata.tbl > $CLCOM

# divide into subpixel
#magnify ldata/w20050414_00059.fits test.fits 2 2 interpo=nearest
cat <<EOF >> $CLCOM
imdel tmpbadpix.pl
magnify $BADPIX tmpbadpix.pl 2 2 fluxcon-

imdel @tdata.tbl
magnify @ldata.tbl @tdata.tbl 2 2 interpo=nearest
EOF

for tdata in `cat tdata.tbl`;do
 echo hedit $tdata bpm tmpbadpix.pl add+ ver- >> $CLCOM
done
echo logout >> $CLCOM

runiraf $CLCOM $CLLOG $CLLOGCOM

# Shift & Add
cat <<EOF > $CLCOM
imdel $FINAL
#imcombine @tdata.tbl $FINAL combine=median reject=sigclip offsets=offset.tbl
imcombine @tdata.tbl $FINAL combine=average reject=sigclip offsets=offset.tbl masktype=goodvalue maskvalue=0
disp $FINAL 2 fi+
#disp final.fits 1 fi+
logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM

echo "Final frame saved as $FINAL"

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'


