#! /usr/bin/sh

if [ $# != 5 ]; then
    echo "Usage : "$0" (YYMMDD) (Detector ID) (Start No.) (No. of Frames) (Header Name)"
    exit 1
fi

DATE=$1
DETECTOR=$2
IS=$3
N=$4
HEAD=$5


# Check Architecture
ARCH=`uname`
#uname -a >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

SKYFLAT=${HEAD}_flat${DETECTOR}.fits
SKY=${HEAD}_sky${DETECTOR}.fits
DARK=dark${DETECTOR}.fits

DATAPATH=$DATA/wfcam${DETECTOR}/${DATE}

TDATAPATH=./tdata
LDATAPATH=./ldata

if [ ! -d $LDATAPATH ];then
 mkdir $LDATAPATH
fi

if [ ! -d $TDATAPATH ];then
 mkdir $TDATAPATH
fi

case $DETECTOR in 
1) DETECTORC=w ;;
2) DETECTORC=x ;;
3) DETECTORC=y ;;
4) DETECTORC=z ;;
esac

gawk -v path=$DATAPATH -v date=$DATE -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl

gawk -v path=$TDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > tdata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

echo imdel @ldata.tbl | cl
echo imdel @tdata.tbl | cl
paste data.tbl ldata.tbl | gawk -v dark=$DARK '{printf("./bin/fwfcam %s\"[1]\" %s %s\n",$1,$2,dark)}' | csh
gawk '{printf("hedit %s ctype1 RA ver-\nhedit %s ctype2 DEC ver-\n",$1,$1)}'  ldata.tbl | cl

cat << EOF | cl
 del tmp.dat
 imstati @ldata-1.tbl upper=5500 lower=2500 format- > tmp.dat
EOF
gawk '{s+=$3;a[NR]=$3}END{s=s/NR;for(i=1;i<=NR;i++)print(a[i]/s)}' tmp.dat > tmp2.dat

echo imarith @ldata.tbl / @tmp2.dat @ldata.tbl | cl

cat <<EOF | cl
imdel $SKY
imcombine @ldata.tbl $SKY combine=median reject=sigclip
#imdel @ldata.tbl
EOF

cat <<EOF | cl
del tmp.dat
imstati $SKY format- > tmp.dat
EOF

IMSTATUPPER=`gawk '{print($3+$4*3)}' tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' tmp.dat`

cat <<EOF | cl
del tmp.dat
imstati $SKY format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > tmp.dat
imdel $SKYFLAT
imarith $SKY / @tmp.dat $SKYFLAT
EOF

# Fix badpix
cat << EOF | cl
imdel tmp.imh
imcopy $SKYFLAT tmp.imh
imreplace tmp.imh -1 upper=0.01 
imreplace tmp.imh 0 lower=0
imreplace tmp.imh 1 upper=-0.5
fixpix $SKYFLAT tmp.imh 
EOF

echo "Sky frame saved as $SKY"
echo "Skyflat frame saved as $SKYFLAT"

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'

